@@ -14,11 +14,11 @@
 from cinderclient import client as cc
 from cinderclient import exceptions
 from keystoneauth1 import exceptions as ks_exceptions
+from oslo_config import cfg
 from oslo_log import log as logging
 
 from heat.common import exception
 from heat.common.i18n import _
-from heat.common.i18n import _LI
 from heat.engine.clients import client_plugin
 from heat.engine.clients import os as os_client
 from heat.engine import constraints
@@ -62,6 +62,8 @@ class CinderClientPlugin(client_plugin.ClientPlugin):
             'extensions': extensions,
             'interface': self.interface,
             'service_type': self.service_type,
+            'region_name': self._get_region_name(),
+            'connect_retries': cfg.CONF.client_retry_limit,
             'http_log_debug': self._get_client_option(CLIENT_NAME,
                                                       'http_log_debug')
         }
@@ -128,24 +130,29 @@ class CinderClientPlugin(client_plugin.ClientPlugin):
         return (isinstance(ex, exceptions.ClientException) and
                 ex.code == 409)
 
-    def check_detach_volume_complete(self, vol_id):
+    def check_detach_volume_complete(self, vol_id, server_id=None):
         try:
             vol = self.client().volumes.get(vol_id)
         except Exception as ex:
             self.ignore_not_found(ex)
             return True
 
-        if vol.status in ('in-use', 'detaching'):
-            LOG.debug('%s - volume still in use' % vol_id)
+        server_ids = [
+            a['server_id'] for a in vol.attachments if 'server_id' in a]
+        if server_id and server_id not in server_ids:
+            return True
+
+        if vol.status in ('in-use', 'detaching', 'reserved'):
+            LOG.debug('%s - volume still in use', vol_id)
             return False
 
-        LOG.debug('Volume %(id)s - status: %(status)s' % {
+        LOG.debug('Volume %(id)s - status: %(status)s', {
             'id': vol.id, 'status': vol.status})
 
         if vol.status not in ('available', 'deleting'):
             LOG.debug("Detachment failed - volume %(vol)s "
-                      "is in %(status)s status" % {"vol": vol.id,
-                                                   "status": vol.status})
+                      "is in %(status)s status",
+                      {"vol": vol.id, "status": vol.status})
             raise exception.ResourceUnknownStatus(
                 resource_status=vol.status,
                 result=_('Volume detachment failed'))
@@ -154,21 +161,21 @@ class CinderClientPlugin(client_plugin.ClientPlugin):
 
     def check_attach_volume_complete(self, vol_id):
         vol = self.client().volumes.get(vol_id)
-        if vol.status in ('available', 'attaching'):
+        if vol.status in ('available', 'attaching', 'reserved'):
             LOG.debug("Volume %(id)s is being attached - "
-                      "volume status: %(status)s" % {'id': vol_id,
-                                                     'status': vol.status})
+                      "volume status: %(status)s",
+                      {'id': vol_id, 'status': vol.status})
             return False
 
         if vol.status != 'in-use':
             LOG.debug("Attachment failed - volume %(vol)s is "
-                      "in %(status)s status" % {"vol": vol_id,
-                                                "status": vol.status})
+                      "in %(status)s status",
+                      {"vol": vol_id, "status": vol.status})
             raise exception.ResourceUnknownStatus(
                 resource_status=vol.status,
                 result=_('Volume attachment failed'))
 
-        LOG.info(_LI('Attaching volume %(id)s complete'), {'id': vol_id})
+        LOG.info('Attaching volume %(id)s complete', {'id': vol_id})
         return True
 
 
